#
# Copyright (C) 2015  Ilya Markov
#
# Full copyright notice can be found in LICENSE.
#
import sys
from enum import Enum

import ainl2015_utils
from pyclick.click_models.ClickModel import ClickModel
from pyclick.click_models.Inference import EMInference
from pyclick.click_models.Param import ParamEM
from pyclick.click_models.ParamContainer import QueryDocumentParamContainer, RankParamContainer
from pyclick.click_models.Evaluation import LogLikelihood, Perplexity


class PBMAttr(ParamEM):
    def update(self, search_session, rank, session_params):
        """
        Updates the value of the parameter based on the observed search session.
        Note that this parameter corresponds to a search result at the given rank.

        :param search_session: The observed search session.
        :param rank: The rank of a search result, which corresponds to the current parameter.
        """
        attr = session_params[rank][PBM.param_names.attr].value()
        exam = session_params[rank][PBM.param_names.exam].value()


class PBMExam(ParamEM):
    def update(self, search_session, rank, session_params):
        """
        Updates the value of the parameter based on the observed search session.
        Note that this parameter corresponds to a search result at the given rank.

        :param search_session: The observed search session.
        :param rank: The rank of a search result, which corresponds to the current parameter.
        """
        attr = session_params[rank][PBM.param_names.attr].value()
        exam = session_params[rank][PBM.param_names.exam].value()


class PBM(ClickModel):
    param_names = Enum('PBMParamNames', 'TODO: XXX')
    """The names of the PBM parameters."""

    def __init__(self):
        self._inference = None
        self.params = None

    def get_full_click_probs(self, search_session):
        """
        Returns a list of full click probabilities P(C = 1) for all results in the given search session.
        """
        click_probs = []
        return click_probs
        
    def get_conditional_click_probs(self, search_session):
        """
        Returns a list of click probabilities conditioned
        on the observed clicks in the given search session.
        In particular, for a result at rank k calculates the following probability:
        P(C_k | C_1, C_2, ..., C_k-1),
        where C_i is 1 if there is a click on the i-th result
        in the given search session and 0 otherwise.
        """
        click_probs = []
        return click_probs


if __name__ == "__main__":
    if len(sys.argv) < 3:
        print "USAGE: %s <dataset> <sessions_max>" % sys.argv[0]
        print "\tdataset - the path to the dataset from Yandex Relevance Prediction Challenge"
        print "\tsessions_max - the maximum number of one-query search sessions to consider"
        print ""
        sys.exit(1)

    search_sessions_path = sys.argv[1]
    search_sessions_num = int(sys.argv[2])
    train_sessions, train_queries, test_sessions, test_queries = ainl2015_utils.get_train_test(
        search_sessions_path, search_sessions_num
    )

    print "Training on %d search sessions (%d unique queries)." % (len(train_sessions), len(train_queries))
    print "Testing on %d search sessions (%d unique queries)." % (len(test_sessions), len(test_queries))

    click_model = PBM()
    click_model.train(train_sessions)
    print click_model

    # perplexity = Perplexity()
    # perplexity.evaluate(click_model, test_sessions)
    #
    # loglikelihood = LogLikelihood()
    # loglikelihood.evaluate(click_model, test_sessions)

